;***************************************************************************
;
; ATtiny13 microdds square wave generator, Stroboscope
; linear frequency scale, duty cycle is constant, e.g PB2: 12.5% 
; frequency 15 - 620 Hz, could be higher, change ADC prescaler
; connect a transistor driver and a bright led and it's done
; 
;  based on ATtiny15 GS 2009
;  This program is free software; you can redistribute it and/or
;  modify it under the terms of the GNU General Public License.
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; 
;
;***************************************************************************
; The timing is adapted for 9.6 MHz 
; clock prescaler set to 0 in reset so we have 9.6 MHz clock
;***************************************************************************
;
;		Pinout ATtiny13/ATtiny13V   8-PDIP/SOIC
;
;(PCINT5/RESET/ADC0/dW)	PB5	VCC                             
;(PCINT3/CLKI/ADC3) 	PB3	PB2 (SCK/ADC1/T0/PCINT2)        
;(PCINT4/ADC2) 		PB4	PB1 (MISO/AIN1/OC0B/INT0/PCINT1)
;			GND	PB0 (MOSI/AIN0/OC0A/PCINT0)     
;***************************************************************************
;
;Description
;
; very poor-mans DDS Synthesizer / Freq Generator with linear scale
; 
; Target = ATtiny13 
; 
;***************************************************************************
;
; PB4: input  ADC2 0-5 Volt, to set frequency
; PB0: output 50% duty cycle (hi) 
; PB1: output 25%
; PB2: output 12.5% 
; PB3: output 6% 
; 
;***************************************************************************
;
; Output frequency (using 16 bit accumulator) :
;
;   	f = deltaPhase * fClock/2^16
;   	fClock is the CPU clock divided by the
;	number of cycles to output the data 
;	cycles ADC 13 * 16 + Loop 14 = 222 -> fClock=43kHz
;	fmax = 2^10 *43 / 2^16 = 0.67 kHz
;	delta1 and delta2 are updated by ADC
;	it's a 16 bit accumulator
;
.DEVICE ATtiny13	;for gavrasm 3.3

.equ	DDRB_config	= 0b00001111
.equ	portB_config	= 0b00001010 	; 
.equ	ADC_poti	= 0b00000010	; frequency set with ADC2 PB4 (poti)

; r0 used
.def	temp 		= r16
.def	ADCstart	= r17

.def 	Delta1 		= r26		; deltaPhase
.def 	Delta2 		= r27

.def 	Dds1 		= r29		; 2 byte DDS accumulator
.def 	Dds2 		= r30
.def 	ZL 		= r30
.def 	ZH 		= r31

.cseg
.org 0
; Reset-vector to adress 0000
		rjmp	reset	

;******************************************************************************
; code
;******************************************************************************

reset:	
		ldi	temp, low(RAMEND)
		out	SPL, temp		; setup stack pointer

   		ldi    	temp,0b10000000		; clock prescaler change enable
   		out	CLKPR,temp
    		ldi     temp,0b00000000		; clock prescaler set to 0     
   		out	CLKPR,temp
   		nop				; wait a little when switching
   		nop
   		nop
   		nop

		; Port B 
		ldi	temp,portB_config 	; set output
		out	PORTB,	temp		; 1 = pull-up , 0 = float
		ldi	temp,DDRB_config 	; low 4 bits are output, 1 = output , 0 = input
		out	DDRB,temp       	; to data direction register


		; AD-Converter setup:
						; ADMUX: REFS1 REFS0 ADLAR   MUX2 MUX1 MUX0
		ldi 	temp, ADC_poti		; Ch 2 input=PB4, REFS0,1 =0 Vcc is RefVoltage
		out 	ADMUX, temp
						
							
		ldi	temp,0b11010100 	; ADC single conversion prescale 16:                
		out 	ADCSRA, temp      	; ADCSR: ADEN ADSC ADATE ADIF ADIE ADPS2 ADPS1 ADPS0
		mov	ADCstart, temp

		sbi	AcsR,	Acd		; Disable Comparator                     
		
		; set pulse25 output
		
		ldi	ZH,high(2*pulse25)	; setup Z pointer hi
		ldi	ZL,low(2*pulse25)	; setup Z pointer lo = dds2

		; clear accumulator 

		ldi 	dds1,0x00		; clear accumulator 
		ldi 	dds2,0x00		; clear accumulator 

		; setup adder registers		
		
		ldi 	delta1,0xff		; in ADCL
		ldi 	delta2,0x0f		;    ADCH


;**********************************************************************
; main loop
;
;	dds1,dds2 = r30 is the phase accumulator
;  	delta1,delta2 is the phase adder value determining frequency
;
;	get ADC value
; 	add value to accumulator
;	load byte from table in ROM
;	output byte to port
;	repeat 
;**********************************************************************
;
LOOP1:
						; get ADC value	for frequency setting
		out 	ADCSRA, ADCstart	; start conversion	; 1
wait2:                                  	
		sbis 	ADCSRA, ADIF        	; 1
		rjmp 	wait2			; 2
		in 	delta1, ADCL 		; first low byte 	; 1
		in 	delta2, ADCH 		; high byte 		; 1
						; 622 Hz		
;		lsl	delta1          	
;		rol	delta2			; ADC * 2 
;		lsl	delta1          	
;		rol	delta2			; ADC * 4 
		
		add	dds1,delta1		; 1
		adc	dds2,delta2		; 1
		lpm				; 3
		out	PORTB, r0		; 1
		rjmp	LOOP1			; 2 => 14 cycles  + ADC


;******************************************************************************
; data tables
;******************************************************************************

	; force table to begin at 256 byte boundary

.org 0x100


phsquare:	; 256 step phased squarewave table PB0,1 180 PB0,2 90 degrees
; 1. quadrant 1010 = A
.db	0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A
.db	0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A
.db	0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A
.db	0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A,0x0A
; 2. quadrant 0110 = 6
.db	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06
.db	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06
.db	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06
.db	0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06
; 3. quadrant 0101 = 5
.db	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05
.db	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05
.db	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05
.db	0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05
; 4. quadrant 1001 = 9
.db	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09
.db	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09
.db	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09
.db	0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09

pulse25:	; 256 step 6 12 25 50% duty cycle table
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01
.db	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01
.db	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01
.db	0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01
.db	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03
.db	0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03
.db	0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07
.db	0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f


;******************************************************************************
; end of file	
;******************************************************************************

